# 14-免密登录集成



## 一、接口响应返回值说明



接口响应是否成功，用HTTP状态码标识

| 状态值 | 说明                                              |
| ------ | ------------------------------------------------- |
| 200    | 接口调用成功                                      |
| 401    | Token值错误或已失效，无权访问（一般认为是未登录） |
| 403    | 禁止访问，无权访问该接口                          |
| 500    | 接口异常，返回具体异常信息                        |

500的异常信息示例：

```json

{
  "code": 10010,  //业务状态码，其值，参考状态码列表
  "message": "文件扩展名必须是[jpg,jpeg,gif,png,bmp,tif,pdf],文件大小不能超过10M",   //错误提示信息
  "detailMessage": "OCRFileParamter 参数错误",    //错误详情
  "data": null  //返回的数据体，根据部分接口需求，会返回错误的数据体或者传参的原数据，一般情况下，该值为null
}
```





## 二、访问润普系统，免密访问

**访问地址**:`http://ip:port/view`

**请求方式**:`GET`

**响应数据类型**:`text/html`

**接口描述**:



**请求参数**:


| 参数名称 | 参数说明 |  是否必须 | 数据类型 | 
| -------- |  ----- | -------- | -------- | 
|account|登录账号|true|string|
|timestamp|当前时间戳，精确到毫秒如1657096573998，该参数控制签名有效时长，默认为300秒|true|Long|
| signature |签名参数，生成签名方式如下：<br />1. 将参数字符串（排除signature），重新按照再把英⽂等号连接得到的字符串按参数名称的字典顺序依次使⽤&符号连接，即得到规范化请求字符串newParam<br />2. 将将第⼀步⽣成的字参串newParam,前后加上签名salt,中间由:进⾏分隔⽣成新的字符串如${salt}:${newParam}:${salt}<br />3. 采⽤MD5信息摘要算法,⽣成签名|true|string|



**生成签名代码样例**:

```java
// 获取签名字符串
public static String getSignature(Map<String,Object> params,String salt){
        if(ObjectUtils.isEmpty(params) || StringUtils.isEmpty(salt)){
            return null;
        }
        Map<String,Object> newParams = null;
        if(params instanceof TreeMap){
            newParams = params;
        }else{
            // 根据ASCII排序参数名称
            newParams = new TreeMap<>();
            newParams.putAll(params);
        }
        // 组装为 param1=123&param2=hello&param3=test
        String sortParam = Joiner.on("&").useForNull("").withKeyValueSeparator("=").join(newParams);
        // 打印
        System.out.println(String.format(" ---- 排序后的字符串参数: %s ",sortParam));
        // md5 加密
        return SecureUtil.md5(salt +":" + sortParam +":"+ salt);
    }

    public static void main(String[] args) {

        Map<String,Object> params = new HashMap<>();
        params.put("account","test");
        params.put("timestamp",1607414951726l);
        params.put("extra","other");

        System.out.println(String.format(" ---- 排序前的字符串参数: %s ",Joiner.on("&").useForNull("").withKeyValueSeparator("=").join(params)));
        System.out.println(String.format(" ---- 签名字符串: %s",getSignature(params,"123456")));
    }


/*
样例测试：

salt = 123456
 ---- 排序前的字符串参数: extra=other&account=test&timestamp=1607414951726 
 ---- 排序后的字符串参数: account=test&extra=other&timestamp=1607414951726 
 ---- 签名字符串: 0c29c249b7bc3858875d269dda902bac

*/
```



**响应状态**:


| 状态码 | 说明 | 
| -------- | -------- | 
|200|OK|
|201|Created|
|401|Unauthorized|
|403|Forbidden|
|404|Not Found|
|500|访问错误|

## 状态码列表



| 状态值 | 状态说明                |
| ------ | ----------------------- |
| 20000  | 账号({0})不存在         |
| 20003  | 校验失败                |
| 20026  | 必输参数：{0},不能为空  |
| 20008  | 该用户禁止使用token访问 |
| 20009  | 该角色不存在            |
| 20027  | 签名有效期(timestamp)过期，最长有效期为: {0} 秒            |